import { useRef, useEffect, useState, MouseEvent } from 'react';
import { mouseEventToCanvasPoint } from './Helpers';

interface Props {
  duration: number;
  style: React.CSSProperties;
}

const LogicalHeight = 200;
const MarginX = 0;
const MarginY = 85;
const KnobRadius = 40;

export const SeekBar: React.FC<Props> = (props: Props) => {
  const canvasRef = useRef<HTMLCanvasElement>(null);
  const [position, _setPosition] = useState(100);
  const [cursor, setCursor] = useState('auto');

  const secsToCanvasX = (secs: number, width: number): number => {
    return (secs / props.duration) * width;
  };

  // draw the canvas
  useEffect(() => {
    const canvas = canvasRef.current;
    if (canvas == null) {
      return;
    }

    const ctx = canvas.getContext('2d');
    if (ctx == null) {
      console.error('no seekbar 2d context available');
      return;
    }

    // Set aspect ratio.
    canvas.width = canvas.height * (canvas.clientWidth / canvas.clientHeight);

    ctx.fillStyle = '#333333';
    ctx.fillRect(0, 0, canvas.width, canvas.height);

    ctx.fillStyle = 'black';
    ctx.fillRect(
      MarginX,
      MarginY,
      canvas.width - MarginX * 2,
      canvas.height - MarginY * 2
    );

    const x = secsToCanvasX(position, canvas.width);
    const y = LogicalHeight / 2;
    ctx.beginPath();
    ctx.arc(x, y, KnobRadius, 0, 2 * Math.PI, false);
    ctx.fillStyle = 'red';
    ctx.fill();
  });

  const style = { ...props.style, cursor: cursor };

  // handlers

  const handleMouseMove = (evt: MouseEvent<HTMLCanvasElement>) => {
    const { x: _x, y: y } = mouseEventToCanvasPoint(evt);
    // TODO: improve mouse detection around knob.
    if (y > MarginY && y < LogicalHeight - MarginY) {
      setCursor('pointer');
    } else {
      setCursor('auto');
    }
  };

  return (
    <>
      <canvas
        style={style}
        ref={canvasRef}
        height={LogicalHeight}
        onMouseMove={handleMouseMove}
      ></canvas>
    </>
  );
};
